function []=StabilityTestThermofluidExample()
% Test stability (common Lyapunov function) for switched dynamics of the
% inter-agent dynamics of the thermo-fluid example.
%
% No inputs, outputs.
%
% --
% Supplementary material (simulation examples) to:
%   Distributed Event-based State Estimation (technical report)
%   Event-based State Estimation: An Emulation-based Approach (journal)
%
% Copyright 2015-17 Max Planck Society. All rights reserved.
% 
% Sebastian Trimpe
% Max Planck Institute for Intelligent Systems
% Autonomous Motion Department
% strimpe(at)tuebingen.mpg.de
%
% Revision history
% [01.07.15, ST]    first version
% [04.08.15, ST]    clean up/revision (completed)

disp(' ');
disp('===================================');
disp(' Thermo-fluid simulation example');
disp('===================================');
disp(' ');


%% Model and estimator design
% Run configuration and design functions.
p_sys = ConfigureSystemThermoFluid();  
design = DesignControllerAndObserverThermoFluid(p_sys);

% Model and observer gain
Ad = p_sys.est.A;
Cd = p_sys.est.C;
L = design.L; 

% Permutations
% Generate all possible permutations of the set {1,2,3,4}; that is, all
% possible sending configurations.
per{1} = [];        % no sensor is communicated
per{2} = 1;         % one sensor
per{3} = 2;
per{4} = 3;
per{5} = 4;
per{6} = [1,2];     % two sensors
per{7} = [1,3];
per{8} = [1,4];
per{9} = [2,3];
per{10} = [2,4];
per{11} = [3,4];
per{12} = [1,2,3];  % three sensors
per{13} = [1,2,4];
per{14} = [1,3,4];
per{15} = [2,3,4];
per{16} = [1,2,3,4];    % all (four) sensors


%% Execute stability test
% Stability test for switching system that is given by the estimator
% dynamics under an arbitrary communication pattern (permutations as
% defined above).

disp(' ');
disp(' Stability test inter-agent dynamics');
disp('---------------------------------------');

% Lyapunov candidate
P = diag([500 1 500 1]);    % basically guessed...
disp(' ');
disp('Lyapunov function candidate:  z^T * P * z.  P = ');
disp(P);
disp(' ');

% Closed loop matrices (one for each permutation)
for ip = 1:length(per)
    if isempty(per{ip})
        A_i{ip} = Ad;
    else
        A_i{ip} = (eye(4)-L(:,per{ip})*Cd(per{ip},:))*Ad;
    end;
end;

% Test
testM = [];
for i=1:length(A_i)
    Q_i{i} = P-A_i{i}'*P*A_i{i};    % Q_i must be positive definite for stability
    testM = [testM, eig(Q_i{i})];
    
    % check and display
    testPassed = all(eig(Q_i{i})>eps(100));
    disp(['Mode #',num2str(i),'.  Test passed? ',num2str(testPassed),'.     eig(Q) =  ',num2str(eig(Q_i{i})')]);
end;

% % Check
% disp('All entries must be positive for stability:');
% disp(testM');
% disp(testM'>eps(100));

